@@ -22,104 +22,52 @@
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
+#include <dirent.h>
 #include <sys/fcntl.h>
 
 #include "libbridge.h"
 #include "libbridge_private.h"
 
-#ifdef HAVE_LIBSYSFS
-/* Given two two character "0a" convert it to a byte */
-static unsigned char getoctet(const char *cp) 
+static FILE *fpopen(const char *dir, const char *name)
 {
-	char t[3] = { cp[0], cp[1], 0 };
-	return strtoul(t, NULL, 16);
-}
-
-static struct sysfs_directory *bridge_sysfs_directory(const char *devname,
-						      const char *subname)
-{
-	struct sysfs_directory *sdir;
-	struct sysfs_class_device *dev;
 	char path[SYSFS_PATH_MAX];
 
-	if (!devname)
-		return NULL;
-
-	if (!br_class_net) {
-		dprintf("can't find class_net\n");
-		return NULL;
-	}
-
-	dev = sysfs_get_class_device(br_class_net, (char *) devname);
-	if (!dev) {
-		dprintf("can't find device %s in %s\n", devname, br_class_net->path);
-		return NULL;
-	}
-
-	snprintf(path, SYSFS_PATH_MAX, "%s/%s", dev->path, subname);
-	sdir = sysfs_open_directory(path);
-	if (!sdir)
-		dprintf("can't open directory: %s\n", path);
-	return sdir;
+	snprintf(path, SYSFS_PATH_MAX, "%s/%s", dir, name);
+	return fopen(path, "r");
 }
 
-static void fetch_id(struct sysfs_directory *sdir, const char *name, 
-		     struct bridge_id *id)
+static void fetch_id(const char *dev, const char *name, struct bridge_id *id)
 {
-	struct sysfs_attribute *attr;
+	FILE *f = fpopen(dev, name);
 
-	memset(id, 0, sizeof(id));
-	attr = sysfs_get_directory_attribute(sdir, (char *) name);
-	if (!attr) {
-		dprintf("Can't find attribute %s/%s\n", sdir->path, name);
-		return;
-	}
-
-	if (strlen(attr->value) < 17) 
-		dprintf("Bad format for %s: '%s'\n", name, attr->value);
-	else {
-		const char *cp = attr->value;
-		id->prio[0] = getoctet(cp); cp += 2;
-		id->prio[1] = getoctet(cp); cp += 3;
-		id->addr[0] = getoctet(cp); cp += 2;
-		id->addr[1] = getoctet(cp); cp += 2;
-		id->addr[2] = getoctet(cp); cp += 2;
-		id->addr[3] = getoctet(cp); cp += 2;
-		id->addr[4] = getoctet(cp); cp += 2;
-		id->addr[5] = getoctet(cp);
-	}
-}
-
-/* Get a time value out of sysfs */
-static void fetch_tv(struct sysfs_directory *sdir, const char *name,
-		     struct timeval *tv)
-{
-	struct sysfs_attribute *attr
-		= sysfs_get_directory_attribute(sdir, (char *) name);
-
-	if (!attr) {
-		dprintf("Can't find attribute %s/%s\n", sdir->path, name);
-		memset(tv, 0, sizeof(tv));
-		return;
-	}
-
-	__jiffies_to_tv(tv, strtoul(attr->value, NULL, 0));
+	fscanf(f, "%2hhx%2hhx.%2hhx%2hhx%2hhx%2hhx%2hhx%2hhx",
+	      &id->prio[0], &id->prio[1],
+	      &id->addr[0], &id->addr[1], &id->addr[2],
+	      &id->addr[3], &id->addr[4], &id->addr[5]);
+	fclose(f);
 }
 
 /* Fetch an integer attribute out of sysfs. */
-static int fetch_int(struct sysfs_directory *sdir, const char *name)
+static int fetch_int(const char *dev, const char *name)
 {
-	struct sysfs_attribute *attr
-		= sysfs_get_directory_attribute(sdir, (char *) name);
-	int val = 0;
+	FILE *f = fpopen(dev, name);
+	int value = -1;
 
-	if (!attr) 
-		dprintf("Can't find attribute %s/%s\n", sdir->path, name);
-	else 
-		val = strtol(attr->value, NULL, 0);
-	return val;
+	if (!f) 
+		fprintf(stderr, "%s: %s\n", dev, strerror(errno));
+	else {
+		fscanf(f, "%d", &value);
+		fclose(f);
+	}
+	return value;
+}
+
+/* Get a time value out of sysfs */
+static void fetch_tv(const char *dev, const char *name, 
+		    struct timeval *tv)
+{
+	__jiffies_to_tv(tv, fetch_int(dev, name));
 }
-#endif
 
 /*
  * Convert device name to an index in the list of ports in bridge.
@@ -206,40 +154,43 @@ static int old_get_bridge_info(const char *bridge, struct bridge_info *info)
  */
 int br_get_bridge_info(const char *bridge, struct bridge_info *info)
 {
-#ifndef HAVE_LIBSYSFS
-	return old_get_bridge_info(bridge, info);
-#else
-	struct sysfs_directory *sdir;
+	DIR *dir;
+	char path[SYSFS_PATH_MAX];
 
-	sdir = bridge_sysfs_directory(bridge, SYSFS_BRIDGE_ATTR);
-	if (!sdir) 
-		return old_get_bridge_info(bridge,info);
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/bridge", bridge);
+	dir = opendir(path);
+	if (dir == NULL) {
+		dprintf("path '%s' is not a directory\n", path);
+		goto fallback;
+	}
 
 	memset(info, 0, sizeof(*info));
-	fetch_id(sdir, "root_id", &info->designated_root);
-	fetch_id(sdir, "bridge_id", &info->bridge_id);
-	info->root_path_cost = fetch_int(sdir, "root_path_cost");
-	fetch_tv(sdir, "max_age", &info->max_age);
-	fetch_tv(sdir, "hello_time", &info->hello_time);
-	fetch_tv(sdir, "forward_delay", &info->forward_delay);
-	fetch_tv(sdir, "max_age", &info->bridge_max_age);
-	fetch_tv(sdir, "hello_time", &info->bridge_hello_time);
-	fetch_tv(sdir, "forward_delay", &info->bridge_forward_delay);
-	fetch_tv(sdir, "ageing_time", &info->ageing_time);
-	fetch_tv(sdir, "hello_timer", &info->hello_timer_value);
-	fetch_tv(sdir, "tcn_timer", &info->tcn_timer_value);
-	fetch_tv(sdir, "topology_change_timer", 
+	fetch_id(path, "root_id", &info->designated_root);
+	fetch_id(path, "bridge_id", &info->bridge_id);
+	info->root_path_cost = fetch_int(path, "root_path_cost");
+	fetch_tv(path, "max_age", &info->max_age);
+	fetch_tv(path, "hello_time", &info->hello_time);
+	fetch_tv(path, "forward_delay", &info->forward_delay);
+	fetch_tv(path, "max_age", &info->bridge_max_age);
+	fetch_tv(path, "hello_time", &info->bridge_hello_time);
+	fetch_tv(path, "forward_delay", &info->bridge_forward_delay);
+	fetch_tv(path, "ageing_time", &info->ageing_time);
+	fetch_tv(path, "hello_timer", &info->hello_timer_value);
+	fetch_tv(path, "tcn_timer", &info->tcn_timer_value);
+	fetch_tv(path, "topology_change_timer", 
 		 &info->topology_change_timer_value);;
-	fetch_tv(sdir, "gc_timer", &info->gc_timer_value);
+	fetch_tv(path, "gc_timer", &info->gc_timer_value);
 
-	info->root_port = fetch_int(sdir, "root_port");
-	info->stp_enabled = fetch_int(sdir, "stp_state");
-	info->topology_change = fetch_int(sdir, "topology_change");
-	info->topology_change_detected = fetch_int(sdir, "topology_change_detected");
-	sysfs_close_directory(sdir);
+	info->root_port = fetch_int(path, "root_port");
+	info->stp_enabled = fetch_int(path, "stp_state");
+	info->topology_change = fetch_int(path, "topology_change");
+	info->topology_change_detected = fetch_int(path, "topology_change_detected");
 
+	closedir(dir);
 	return 0;
-#endif
+
+fallback:
+	return old_get_bridge_info(bridge, info);
 }
 
 static int old_get_port_info(const char *brname, const char *port,
@@ -293,36 +244,34 @@ static int old_get_port_info(const char *brname, const char *port,
 int br_get_port_info(const char *brname, const char *port, 
 		     struct port_info *info)
 {
-#ifndef HAVE_LIBSYSFS
-	return old_get_port_info(brname, port, info);
-#else
-	struct sysfs_directory *sdir
-		= bridge_sysfs_directory(port, SYSFS_BRIDGE_PORT_ATTR);
+	DIR *d;
+	char path[SYSFS_PATH_MAX];
 
-	if (!sdir) 
-		return old_get_port_info(brname, port, info);
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/brport", port);
+	d = opendir(path);
+	if (!d)
+		goto fallback;
 
 	memset(info, 0, sizeof(*info));
-	fetch_id(sdir, "designated_root", &info->designated_root);
-	fetch_id(sdir, "designated_bridge", &info->designated_bridge);
-	info->port_no = fetch_int(sdir, "port_no");
-	info->port_id = fetch_int(sdir, "port_id");
-	info->designated_port = fetch_int(sdir, "designated_port");
-	info->path_cost = fetch_int(sdir, "path_cost");
-	info->designated_cost = fetch_int(sdir, "designated_cost");
-	info->state = fetch_int(sdir, "state");
-	info->top_change_ack = fetch_int(sdir, "change_ack");
-	info->config_pending = fetch_int(sdir, "config_pending");
-	fetch_tv(sdir, "message_age_timer", 
-		 &info->message_age_timer_value);
-	fetch_tv(sdir, "forward_delay_timer",
-		 &info->forward_delay_timer_value);
-	fetch_tv(sdir, "hold_timer",
-		 &info->hold_timer_value);
-	sysfs_close_directory(sdir);
+
+	fetch_id(path, "designated_root", &info->designated_root);
+	fetch_id(path, "designated_bridge", &info->designated_bridge);
+	info->port_no = fetch_int(path, "port_no");
+	info->port_id = fetch_int(path, "port_id");
+	info->designated_port = fetch_int(path, "designated_port");
+	info->path_cost = fetch_int(path, "path_cost");
+	info->designated_cost = fetch_int(path, "designated_cost");
+	info->state = fetch_int(path, "state");
+	info->top_change_ack = fetch_int(path, "change_ack");
+	info->config_pending = fetch_int(path, "config_pending");
+	fetch_tv(path, "message_age_timer", &info->message_age_timer_value);
+	fetch_tv(path, "forward_delay_timer", &info->forward_delay_timer_value);
+	fetch_tv(path, "hold_timer", &info->hold_timer_value);
+	closedir(d);
 
 	return 0;
-#endif
+fallback:
+	return old_get_port_info(brname, port, info);
 }
 
 
@@ -330,26 +279,17 @@ static int br_set(const char *bridge, const char *name,
 		  unsigned long value, unsigned long oldcode)
 {
 	int ret;
-#ifdef HAVE_LIBSYSFS
-	struct sysfs_directory *sdir;
-	
-	sdir = bridge_sysfs_directory(bridge, SYSFS_BRIDGE_ATTR);
-	if (sdir) {
-		struct sysfs_attribute *attr;
-		char buf[32];
-		sprintf(buf, "%ld", value);
-		
-		attr = sysfs_get_directory_attribute(sdir, (char *) name);
-		if (attr) 
-			ret = sysfs_write_attribute(attr, buf, strlen(buf));
-		else {
-			ret = -1;
-			errno = EINVAL;
-		}
-		sysfs_close_directory(sdir);
-	} else 
-#endif
-	{
+	char path[SYSFS_PATH_MAX];
+	FILE *f;
+
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/%s", bridge, name);
+
+	f = fopen(path, "w");
+	if (f) {
+		ret = fprintf(f, "%ld\n", value);
+		fclose(f);
+	} else {
+		/* fallback to old ioctl */
 		struct ifreq ifr;
 		unsigned long args[4] = { oldcode, value, 0, 0 };
 		
@@ -400,37 +340,28 @@ static int port_set(const char *bridge, const char *ifname,
 		    const char *name, unsigned long value, 
 		    unsigned long oldcode)
 {
-	int ret, index;
-#ifdef HAVE_LIBSYSFS
-	struct sysfs_directory *sdir;
+	int ret;
+	char path[SYSFS_PATH_MAX];
+	FILE *f;
 
-	sdir = bridge_sysfs_directory(ifname, SYSFS_BRIDGE_PORT_ATTR);
-	if (sdir) {
-		struct sysfs_attribute *attr;
-		char buf[32];
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/brport/%s", ifname, name);
+	f = fopen(path, "w");
+	if (f) {
+		ret = fprintf(f, "%ld\n", value);
+		fclose(f);
+	} else {
+		int index = get_portno(bridge, ifname);
 
-		sprintf(buf, "%ld", value);
-
-		attr = sysfs_get_directory_attribute(sdir, (char *) name);
-		if (attr) 
-			ret = sysfs_write_attribute(attr, buf, strlen(buf));
+		if (index < 0)
+			ret = index;
 		else {
-			ret = -1; 
-			errno = EINVAL;
+			struct ifreq ifr;
+			unsigned long args[4] = { oldcode, index, value, 0 };
+			
+			strncpy(ifr.ifr_name, bridge, IFNAMSIZ);
+			ifr.ifr_data = (char *) &args;
+			ret = ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr);
 		}
-		sysfs_close_directory(sdir);
-	} else
-#endif
-	if ( (index = get_portno(bridge, ifname)) < 0)
-		ret = index;
-
-	else {
-		struct ifreq ifr;
-		unsigned long args[4] = { oldcode, index, value, 0 };
-
-		strncpy(ifr.ifr_name, bridge, IFNAMSIZ);
-		ifr.ifr_data = (char *) &args;
-		ret = ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr);
 	}
 
 	return ret < 0 ? errno : 0;
@@ -458,30 +389,19 @@ static inline void __copy_fdb(struct fdb_entry *ent,
 int br_read_fdb(const char *bridge, struct fdb_entry *fdbs, 
 		unsigned long offset, int num)
 {
-	int i, fd = -1, n;
+	FILE *f;
+	int i, n;
 	struct __fdb_entry fe[num];
-#ifdef HAVE_LIBSYSFS
-	struct sysfs_class_device *dev;
+	char path[SYSFS_PATH_MAX];
 	
 	/* open /sys/class/net/brXXX/brforward */
-	if (br_class_net &&
-	    (dev = sysfs_get_class_device(br_class_net, (char *) bridge))) {
-		char path[SYSFS_PATH_MAX];
-
-		snprintf(path, SYSFS_PATH_MAX, "%s/%s", dev->path, 
-			 SYSFS_BRIDGE_FDB);
-		fd = open(path, O_RDONLY, 0);
-	}
-
-	if (fd != -1) {
-		/* read records from file */
-		lseek(fd, offset*sizeof(struct __fdb_entry), SEEK_SET);
-		n = read(fd, fe, num*sizeof(struct __fdb_entry));
-		if (n > 0)
-			n /= sizeof(struct __fdb_entry);
-	} else
-#endif
-	{
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/brforward", bridge);
+	f = fopen(path, "r");
+	if (f) {
+		fseek(f, offset*sizeof(struct __fdb_entry), SEEK_SET);
+		n = fread(fe, sizeof(struct __fdb_entry), num, f);
+		fclose(f);
+	} else {
 		/* old kernel, use ioctl */
 		unsigned long args[4] = { BRCTL_GET_FDB_ENTRIES,
 					  (unsigned long) fe,
@@ -505,8 +425,5 @@ int br_read_fdb(const char *bridge, struct fdb_entry *fdbs,
 	for (i = 0; i < n; i++) 
 		__copy_fdb(fdbs+i, fe+i);
 
-	if (fd > 0)
-		close(fd);
-	
 	return n;
 }
